@@ -382,6 +382,13 @@ struct MVMREPROps_Attribute {
     MVMint64 (*is_attribute_initialized) (MVMThreadContext *tc, MVMSTable *st,
         void *data, MVMObject *class_handle, MVMString *name,
         MVMint64 hint);
+
+    /* Provided the attribute is a native integer of the architecture's atomic
+     * size, returns an AO_t * referencing it. This is only valid until the
+     * next safepoint. If rebless is called on the object, updates may be lost
+     * although memory safety must not be violated. */
+    AO_t * (*attribute_as_atomic) (MVMThreadContext *tc, MVMSTable *st,
+        void *data, MVMObject *class_handle, MVMString *name);
 };
 struct MVMREPROps_Boxing {
     /* Used with boxing. Sets an integer value, for representations that
@@ -498,6 +505,16 @@ struct MVMREPROps_Positional {
 
     /* Gets the STable representing the declared element type. */
     MVMStorageSpec (*get_elem_storage_spec) (MVMThreadContext *tc, MVMSTable *st);
+
+    /* Provided the array consists of integers of the architecture's atomic
+     * size, gets an AO_t * pointing to that element and valid until the next
+     * safepoint. */
+    AO_t * (*pos_as_atomic) (MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
+        void *data, MVMint64 index);
+
+    /* Multi-dim version of as_atomic. */
+    AO_t * (*pos_as_atomic_multidim) (MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
+        void *data, MVMint64 num_indices, MVMint64 *indices);
 };
 struct MVMREPROps_Associative {
     /* Gets the value at the specified key and places it in the passed
@@ -658,3 +675,10 @@ void MVM_6model_invoke_default(MVMThreadContext *tc, MVMObject *invokee, MVMCall
 void MVM_6model_stable_gc_free(MVMThreadContext *tc, MVMSTable *st);
 MVMuint64 MVM_6model_next_type_cache_id(MVMThreadContext *tc);
 void MVM_6model_never_repossess(MVMThreadContext *tc, MVMObject *obj);
+
+MVM_STATIC_INLINE char *MVM_6model_get_debug_name(MVMThreadContext *tc, MVMObject *obj) {
+    return STABLE(obj)->debug_name ? STABLE(obj)->debug_name : "";
+}
+MVM_STATIC_INLINE char *MVM_6model_get_stable_debug_name(MVMThreadContext *tc, MVMSTable *stable) {
+    return stable->debug_name ? stable->debug_name : "";
+}
