@@ -31,17 +31,13 @@ struct MVMStaticFrameBody {
      * the VM instance wide field for this. */
     MVMuint32 instrumentation_level;
 
-    /* Rough call count. May be hit up by multiple threads, and lose the odd
-     * count, but that's fine; it's just a rough indicator, used to make
-     * decisions about optimization. */
-    MVMuint32 invocations;
-
-    /* Number of times we should invoke before spesh applies. */
-    MVMuint32 spesh_threshold;
-
-    /* Specializations array, if there are any. */
-    MVMSpeshCandidate *spesh_candidates;
-    MVMuint32          num_spesh_candidates;
+    /* Specialization-related information. Attached when a frame is first
+     * verified. Held in a separate object rather than the MVMStaticFrame
+     * itself partly to decrease the size of this object for frames that
+     * we never even call, but also because we sample nursery objects and
+     * they end up in the statistics; forcing the static frame into the
+     * inter-generational roots leads to a lot more marking work. */
+    MVMStaticFrameSpesh *spesh;
 
     /* The size in bytes to allocate for the lexical environment. */
     MVMuint32 env_size;
@@ -49,21 +45,40 @@ struct MVMStaticFrameBody {
     /* The size in bytes to allocate for the work and arguments area. */
     MVMuint32 work_size;
 
+    /* Count of lexicals. */
+    MVMuint32 num_lexicals;
+
+    /* Inital contents of the work area, copied into place to make sure we have
+     * VMNulls in all the object slots. */
+    MVMRegister *work_initial;
+
     /* The size of the bytecode. */
     MVMuint32 bytecode_size;
 
     /* Count of locals. */
     MVMuint32 num_locals;
 
-    /* Count of lexicals. */
-    MVMuint32 num_lexicals;
-
     /* Frame exception handlers information. */
     MVMFrameHandler *handlers;
 
     /* The number of exception handlers this frame has. */
     MVMuint32 num_handlers;
 
+    /* Is the frame full deserialized? */
+    MVMuint8 fully_deserialized;
+
+    /* Is the frame a thunk, and thus hidden to caller/outer? */
+    MVMuint8 is_thunk;
+
+    /* Does the frame have an exit handler we need to run? */
+    MVMuint8 has_exit_handler;
+
+    /* Should we allocate the frame directly on the heap? Doing so may avoid
+     * needing to promote it there later. Set by measuring the number of times
+     * the frame is promoted to the heap relative to the number of times it is
+     * invoked, and then only pre-specialization. */
+    MVMuint8 allocate_on_heap;
+
     /* The compilation unit unique ID of this frame. */
     MVMString *cuuid;
 
@@ -76,25 +91,10 @@ struct MVMStaticFrameBody {
     /* the static coderef */
     MVMCode *static_code;
 
-    /* Index into each threadcontext's table of frame pools. */
-    MVMuint32 pool_index;
-
     /* Annotation details */
     MVMuint32              num_annotations;
     MVMuint8              *annotations_data;
 
-    /* Cached instruction offsets */
-    MVMuint8 *instr_offsets;
-
-    /* Does the frame have an exit handler we need to run? */
-    MVMuint8 has_exit_handler;
-
-    /* Is the frame a thunk, and thus hidden to caller/outer? */
-    MVMuint8 is_thunk;
-
-    /* Is the frame full deserialized? */
-    MVMuint8 fully_deserialized;
-
     /* The original bytecode for this frame (before endian swapping). */
     MVMuint8 *orig_bytecode;
 
@@ -110,9 +110,17 @@ struct MVMStaticFrameBody {
     MVMint32 code_obj_sc_dep_idx;
     MVMint32 code_obj_sc_idx;
 
-    /* Profiling/instrumented version of the bytecode, if we're profiling.
-     * Also, a backup of the uninstrumented bytecode in case we turn off
-     * profiling. Same for handlers. */
+    /* Extra profiling/instrumentation state. */
+    MVMStaticFrameInstrumentation *instrumentation;
+};
+struct MVMStaticFrame {
+    MVMObject common;
+    MVMStaticFrameBody body;
+};
+
+/* Extra state that static frames carry when instrumented, so that the
+ * can later be removed again. */
+struct MVMStaticFrameInstrumentation {
     MVMuint8        *instrumented_bytecode;
     MVMuint8        *uninstrumented_bytecode;
     MVMFrameHandler *instrumented_handlers;
@@ -120,10 +128,6 @@ struct MVMStaticFrameBody {
     MVMuint32        uninstrumented_bytecode_size;
     MVMuint32        instrumented_bytecode_size;
 };
-struct MVMStaticFrame {
-    MVMObject common;
-    MVMStaticFrameBody body;
-};
 
 /* Function for REPR setup. */
 const MVMREPROps * MVMStaticFrame_initialize(MVMThreadContext *tc);
