@@ -7,9 +7,9 @@
  */
 
 /* Version information. */
-#define MVM_VERSION "2017.02"
+#define MVM_VERSION "2017.10"
 #define MVM_VERSION_MAJOR 2017
-#define MVM_VERSION_MINOR 02
+#define MVM_VERSION_MINOR 10
 #define MVM_VERSION_PATCH 0
 
 /* Various compiler-specific pragmii. */
@@ -24,35 +24,40 @@
 
 /* Set in case of big-endian sytems.
  * The default byte order is little-endian. */
-#if 1
-#define MVM_BIGENDIAN 1
+#if 0
+#define MVM_BIGENDIAN 0
 #endif
 
 /* pthread_yield() detection */
-#if 1
-#define MVM_HAS_PTHREAD_YIELD 1
+#if 0
+#define MVM_HAS_PTHREAD_YIELD 0
 #endif
 
 /* How this compiler does static inline functions. */
 #define MVM_STATIC_INLINE static __inline__
 
-#if 0
+#if 1
 #define MVM_CAN_UNALIGNED_INT32
 #endif
 
-#if 0
+#if 1
 #define MVM_CAN_UNALIGNED_INT64
 #endif
 
-#if 0
+#if 1
 #define MVM_CAN_UNALIGNED_NUM64
 #endif
 
 #define MVM_PTR_SIZE 8
 
+
 #if 1
 #define MVM_BOOL _Bool
 #endif
 
 /* Should we translate \n to \r\n on output? */
 #define MVM_TRANSLATE_NEWLINE_OUTPUT 0
+
+#define MVM_JIT_ARCH MVM_JIT_ARCH_X64
+#define MVM_JIT_PLATFORM MVM_JIT_PLATFORM_POSIX
+
