@@ -10,10 +10,13 @@ struct MVMArgProcContext {
     /* The arguments. */
     MVMRegister *args;
 
-    /* Bytemap of indexes of used nameds, so the
-     * named slurpy knows which ones not to grab.
-     * XXX cache and free this at the proper times. */
-    MVMuint8 *named_used;
+    /* Indexes of used nameds. If named_used_size is less than or equal to
+     * 64, it will be a bit field. Otherwise, it will be a pointer to a
+     * byte array. */
+    union {
+        MVMuint8 *byte_array;
+        MVMuint64 bit_field;
+    } named_used;
     MVMuint16 named_used_size;
 
     /* The total argument count (including 2 for each
@@ -28,45 +31,38 @@ struct MVMArgProcContext {
 };
 
 /* Expected return type flags. */
-typedef enum {
-    /* Argument is an object. */
-    MVM_RETURN_VOID = 0,
-
-    /* Argument is an object. */
-    MVM_RETURN_OBJ = 1,
-
-    /* Argument is a native integer, signed. */
-    MVM_RETURN_INT = 2,
-
-    /* Argument is a native floating point number. */
-    MVM_RETURN_NUM = 4,
-
-    /* Argument is a native NFG string (MVMString REPR). */
-    MVM_RETURN_STR = 8,
-} MVMReturnType;
+typedef MVMuint8 MVMReturnType;
+#define MVM_RETURN_VOID     0
+#define MVM_RETURN_OBJ      1
+#define MVM_RETURN_INT      2
+#define MVM_RETURN_NUM      4
+#define MVM_RETURN_STR      8
 
 /* Struct used for returning information about an argument. */
 struct MVMArgInfo {
     MVMRegister arg;
     MVMCallsiteEntry   flags;
     MVMuint8           exists;
+    MVMuint16          arg_idx; /* Set only for nameds, obvious for pos */
 };
 
 /* Argument processing context handling. */
 void MVM_args_proc_init(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMCallsite *callsite, MVMRegister *args);
-void MVM_args_proc_cleanup_for_cache(MVMThreadContext *tc, MVMArgProcContext *ctx);
 void MVM_args_proc_cleanup(MVMThreadContext *tc, MVMArgProcContext *ctx);
 void MVM_args_checkarity(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint16 min, MVMuint16 max);
 void MVM_args_checkarity_for_jit(MVMThreadContext *tc, MVMuint16 min, MVMuint16 max);
 MVMCallsite * MVM_args_copy_callsite(MVMThreadContext *tc, MVMArgProcContext *ctx);
-MVMCallsite * MVM_args_proc_to_callsite(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint8 *owns_callsite);
-MVMCallsite * MVM_args_prepare(MVMThreadContext *tc, MVMCompUnit *cu, MVMint16 callsite_idx);
+MVMCallsite * MVM_args_copy_uninterned_callsite(MVMThreadContext *tc, MVMArgProcContext *ctx);
 MVM_PUBLIC MVMObject * MVM_args_use_capture(MVMThreadContext *tc, MVMFrame *f);
 MVM_PUBLIC MVMObject * MVM_args_save_capture(MVMThreadContext *tc, MVMFrame *f);
+void MVM_args_marked_named_used(MVMThreadContext *tc, MVMuint32 idx);
+void MVM_args_throw_named_unused_error(MVMThreadContext *tc, MVMString *name);
 
 /* Argument access by position. */
-MVMArgInfo MVM_args_get_pos_obj(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos, MVMuint8 required);
-MVMArgInfo MVM_args_get_pos_int(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos, MVMuint8 required);
+MVMObject * MVM_args_get_required_pos_obj(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos);
+MVMArgInfo MVM_args_get_optional_pos_obj(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos);
+MVMint64 MVM_args_get_required_pos_int(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos);
+MVMArgInfo MVM_args_get_optional_pos_int(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos);
 MVMArgInfo MVM_args_get_pos_num(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos, MVMuint8 required);
 MVMArgInfo MVM_args_get_pos_str(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos, MVMuint8 required);
 MVMArgInfo MVM_args_get_pos_uint(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint32 pos, MVMuint8 required);
