@@ -22,31 +22,11 @@ typedef enum {
     MVMGCStatus_STOLEN = 3
 } MVMGCStatus;
 
-/* To manage memory more efficiently, we cache MVMFrame instances.
- * The initial frame pool table size sets the initial guess at the
- * number of different types of frame (that is, an MVMStaticFrame)
- * that we'll encounter and cache. If we do deep recursion, we run
- * the risk of caching an enormous number of frames, so the length
- * limit sets how many frames of a given static frame type we will
- * keep around. */
-#define MVMInitialFramePoolTableSize    64
-#define MVMFramePoolLengthLimit         64
-
 /* Information associated with an executing thread. */
 struct MVMThreadContext {
-    /* The current allocation pointer, where the next object to be allocated
-     * should be placed. */
-    void *nursery_alloc;
-
-    /* The end of the space we're allowed to allocate to. */
-    void *nursery_alloc_limit;
-
-    /* This thread's GC status. */
-    AO_t gc_status;
-
-    /* Non-zero is we should allocate in gen2; incremented/decremented as we
-     * enter/leave a region wanting gen2 allocation. */
-    MVMuint32 allocate_in_gen2;
+    /************************************************************************
+     * Information about this thread
+     ************************************************************************/
 
     /* Internal ID of the thread. */
     MVMuint32 thread_id;
@@ -54,57 +34,16 @@ struct MVMThreadContext {
     /* Thread object representing the thread. */
     MVMThread *thread_obj;
 
-    /* Pointer to where the interpreter's current opcode is stored. */
-    MVMuint8 **interp_cur_op;
-
-    /* Pointer to where the interpreter's bytecode start pointer is stored. */
-    MVMuint8 **interp_bytecode_start;
-
-    /* Pointer to where the interpreter's base of the current register
-     * set is stored. */
-    MVMRegister **interp_reg_base;
-
-    /* Pointer to where the interpreter's current compilation unit pointer
-     * is stored. */
-    MVMCompUnit **interp_cu;
-
-    /* First call stack memory region, so we can traverse them for cleanup. */
-    MVMCallStackRegion *stack_first;
-
-    /* Current call stack region, which the next frame will be allocated in. */
-    MVMCallStackRegion *stack_current;
-
-    /* The frame we're currently executing. */
-    MVMFrame *cur_frame;
-
-    /* The frame lying at the base of the current thread. */
-    MVMFrame *thread_entry_frame;
-
-    /* libuv event loop */
-    uv_loop_t *loop;
-
-    /* The usecapture op can, without allocating, have a way to talk about the
-     * arguments of the current call. This is the (pre-thread) object that is
-     * used by that op. */
-    MVMObject *cur_usecapture;
-
-    /* Linked list of exception handlers that we're currently executing, topmost
-     * one first in the list. */
-    MVMActiveHandler *active_handlers;
-
-    /* Result object of the last-run exception handler. */
-    MVMObject *last_handler_result;
-
-    /* Last payload made available in a payload-goto exception handler. */
-    MVMObject *last_payload;
-
-    /* Mutex that must be released if we throw an exception. Used in places
-     * like I/O, which grab a mutex but may throw an exception. */
-    uv_mutex_t *ex_release_mutex;
-
     /* The VM instance that this thread belongs to. */
     MVMInstance *instance;
 
+    /* The number of locks the thread is holding. */
+    MVMint64 num_locks;
+
+    /************************************************************************
+     * Garbage collection and memory management
+     ************************************************************************/
+
     /* Start of fromspace, the place we're copying objects from during a
      * copying collection or processing dead objects that need to do extra
      * resource release afterwards. */
@@ -114,19 +53,31 @@ struct MVMThreadContext {
      * allocate new ones. */
     void *nursery_tospace;
 
+    /* The current allocation pointer, where the next object to be allocated
+     * should be placed. */
+    void *nursery_alloc;
+
+    /* The end of the space we're allowed to allocate to. */
+    void *nursery_alloc_limit;
+
+    /* This thread's GC status. */
+    AO_t gc_status;
+
     /* The second GC generation allocator. */
     MVMGen2Allocator *gen2;
 
+    /* The current sizes of the nursery fromspace/tospace for this thread, in
+     * bytes. Used to handle growing it over time depending on usage. */
+    MVMuint32 nursery_fromspace_size;
+    MVMuint32 nursery_tospace_size;
+
+    /* Non-zero is we should allocate in gen2; incremented/decremented as we
+     * enter/leave a region wanting gen2 allocation. */
+    MVMuint32 allocate_in_gen2;
+
     /* Number of bytes promoted to gen2 in current GC run. */
     MVMuint32 gc_promoted_bytes;
 
-    /* Memory buffer pointing to the last thing we serialized, intended to go
-     * into the next compilation unit we write. Also the serialized string
-     * heap, which will be used to seed the compilation unit string heap. */
-    MVMint32      serialized_size;
-    char         *serialized;
-    MVMObject    *serialized_string_heap;
-
     /* Temporarily rooted objects. This is generally used by code written in
      * C that wants to keep references to objects. Since those may change
      * if the code in question also allocates, there is a need to register
@@ -164,10 +115,117 @@ struct MVMThreadContext {
     MVMuint32        gc_work_size;
     MVMuint32        gc_work_count;
 
-    /* Pool of Lexotics for various static frames, held per thread since the
-     * result being returned is per thread. */
-    MVMLexotic **lexotic_cache;
-    MVMuint32    lexotic_cache_size;
+    /* Per-thread fixed size allocator state. */
+    MVMFixedSizeAllocThread *thread_fsa;
+
+    /************************************************************************
+     * Interpreter state
+     ************************************************************************/
+
+    /* Pointer to where the interpreter's current opcode is stored. */
+    MVMuint8 **interp_cur_op;
+
+    /* Pointer to where the interpreter's bytecode start pointer is stored. */
+    MVMuint8 **interp_bytecode_start;
+
+    /* Pointer to where the interpreter's base of the current register
+     * set is stored. */
+    MVMRegister **interp_reg_base;
+
+    /* Pointer to where the interpreter's current compilation unit pointer
+     * is stored. */
+    MVMCompUnit **interp_cu;
+
+    /* Jump buffer, used when an exception is thrown from C-land and we need
+     * to fall back into the interpreter. These things are huge, so put it
+     * near the end to keep the hotter stuff on the same cacheline. */
+    jmp_buf interp_jump;
+
+    /************************************************************************
+     * Frames, call stack, and exception state
+     ************************************************************************/
+
+    /* The frame we're currently executing. */
+    MVMFrame *cur_frame;
+
+    /* The frame lying at the base of the current thread. */
+    MVMFrame *thread_entry_frame;
+
+    /* First call stack memory region, so we can traverse them for cleanup. */
+    MVMCallStackRegion *stack_first;
+
+    /* Current call stack region, which the next frame will be allocated in. */
+    MVMCallStackRegion *stack_current;
+
+    /* Linked list of exception handlers that we're currently executing, topmost
+     * one first in the list. */
+    MVMActiveHandler *active_handlers;
+
+    /* Result object of the last-run exception handler. */
+    MVMObject *last_handler_result;
+
+    /* Last payload made available in a payload-goto exception handler. */
+    MVMObject *last_payload;
+
+    /************************************************************************
+     * Specialization and JIT compilation
+     ************************************************************************/
+
+    /* Frame sequence numbers in order to cheaply identify the place of a frame
+     * in the call stack */
+    MVMint32 current_frame_nr;
+    MVMint32 next_frame_nr;
+
+    /* This thread's current spesh log to write in to, if there curently is
+     * one. */
+    MVMSpeshLog *spesh_log;
+
+    /* How many spesh logs we can produce, inclusive of the current one.
+     * Ensures the spesh worker isn't overwhelmed with data before it has a
+     * change to produce some specializations. */
+    AO_t spesh_log_quota;
+
+    /* The spesh stack simulation, perserved between processing logs. */
+    MVMSpeshSimStack *spesh_sim_stack;
+
+    /* We try to do better at OSR by creating a fresh log when we enter a new
+     * compilation unit. However, for things that EVAL or do a ton of BEGIN,
+     * this does more harm than good. Use this to throttle it back. */
+    MVMuint32 num_compunit_extra_logs;
+
+    /* The current specialization correlation ID, used in logging. */
+    MVMuint32 spesh_cid;
+
+#if MVM_GC_DEBUG
+    /* Whether we are currently in the specializer. Used to catch GC runs that
+     * take place at times they never should. */
+    MVMint32 in_spesh;
+#endif
+
+    /* State to cheaply determine if we should look again for the availability
+     * of optimzied code at an OSR point. When the current state seen by the
+     * interpreter of frame number of spesh candidates matches, we know there
+     * was no change since the last OSR point. */
+    MVMint32 osr_hunt_frame_nr;
+    MVMint32 osr_hunt_num_spesh_candidates;
+
+    /************************************************************************
+     * Per-thread state held by assorted VM subsystems
+     ************************************************************************/
+
+    /* libuv event loop */
+    uv_loop_t *loop;
+
+    /* Mutex that must be released if we throw an exception. Used in places
+     * like I/O, which grab a mutex but may throw an exception. */
+    uv_mutex_t *ex_release_mutex;
+
+    /* Memory buffer pointing to the last thing we serialized, intended to go
+     * into the next compilation unit we write. Also the serialized string
+     * heap, which will be used to seed the compilation unit string heap. */
+    MVMint32      serialized_size;
+    char         *serialized;
+    MVMObject    *serialized_string_heap;
 
     /* Serialization context write barrier disabled depth (anything non-zero
      * means disabled). */
@@ -188,16 +246,11 @@ struct MVMThreadContext {
     /* Random number generator state. */
     MVMuint64 rand_state[2];
 
-    /* Jump buffer, used when an exception is thrown from C-land and we need
-     * to fall back into the interpreter. These things are huge, so put it
-     * near the end to keep the hotter stuff on the same cacheline. */
-    jmp_buf interp_jump;
-
     /* NFA evaluator memory cache, to avoid many allocations; see NFA.c. */
-    MVMint64 *nfa_done;
-    MVMint64 *nfa_curst;
-    MVMint64 *nfa_nextst;
-    MVMint64  nfa_alloc_states;
+    MVMuint32 *nfa_done;
+    MVMuint32 *nfa_curst;
+    MVMuint32 *nfa_nextst;
+    MVMint64   nfa_alloc_states;
     MVMint64 *nfa_fates;
     MVMint64  nfa_fates_len;
     MVMint64 *nfa_longlit;
@@ -207,25 +260,11 @@ struct MVMThreadContext {
     MVMint64 *multi_dim_indices;
     MVMint64  num_multi_dim_indices;
 
-    /* The number of locks the thread is holding. */
-    MVMint64 num_locks;
-
     /* Profiling data collected for this thread, if profiling is on. */
     MVMProfileThreadData *prof_data;
-
-    /* Frame sequence numbers in order to cheaply identify the place of a frame
-     * in the call stack */
-    MVMint32 current_frame_nr;
-    MVMint32 next_frame_nr;
-
-#if MVM_GC_DEBUG
-    /* Whether we are currently in the specializer. Used to catch GC runs that
-     * take place at times they never should. */
-    MVMint32 in_spesh;
-#endif
 };
 
-MVMThreadContext * MVM_tc_create(MVMInstance *instance);
+MVMThreadContext * MVM_tc_create(MVMThreadContext *parent, MVMInstance *instance);
 void MVM_tc_destroy(MVMThreadContext *tc);
 void MVM_tc_set_ex_release_mutex(MVMThreadContext *tc, uv_mutex_t *mutex);
 void MVM_tc_release_ex_release_mutex(MVMThreadContext *tc);
