@@ -13,10 +13,6 @@
 #define MVM_PIPE_MERGED_OUT_ERR 512
 
 MVMObject * MVM_proc_getenvhash(MVMThreadContext *tc);
-MVMint64 MVM_proc_shell(MVMThreadContext *tc, MVMString *cmd, MVMString *cwd, MVMObject *env,
-        MVMObject *in, MVMObject *out, MVMObject *err, MVMint64 flags);
-MVMint64 MVM_proc_spawn(MVMThreadContext *tc, MVMObject *argv, MVMString *cwd, MVMObject *env,
-        MVMObject *in, MVMObject *out, MVMObject *err, MVMint64 flags);
 MVMObject * MVM_proc_spawn_async(MVMThreadContext *tc, MVMObject *queue, MVMObject *args,
          MVMString *cwd, MVMObject *env, MVMObject *callbacks);
 void MVM_proc_kill_async(MVMThreadContext *tc, MVMObject *handle, MVMint64 signal);
@@ -29,6 +25,7 @@ MVMint64 MVM_proc_time_i(MVMThreadContext *tc);
 MVMObject * MVM_proc_clargs(MVMThreadContext *tc);
 MVMnum64 MVM_proc_time_n(MVMThreadContext *tc);
 MVMString * MVM_executable_name(MVMThreadContext *tc);
+MVMObject * MVM_proc_getrusage(MVMThreadContext *tc);
 
 #ifdef _WIN32
 #include <wchar.h>
