@@ -94,6 +94,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "gc/collect.h"
 #include "gc/debug.h"
 #include "gc/wb.h"
+#include "core/vector.h"
 #include "core/threadcontext.h"
 #include "core/instance.h"
 #include "core/interp.h"
@@ -110,6 +111,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "core/threads.h"
 #include "core/hll.h"
 #include "core/loadbytecode.h"
+#include "core/bitmap.h"
 #include "math/num.h"
 #include "core/coerce.h"
 #include "core/ext.h"
@@ -145,19 +147,26 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "spesh/args.h"
 #include "spesh/facts.h"
 #include "spesh/optimize.h"
+#include "spesh/dead_bb_elimination.h"
 #include "spesh/deopt.h"
 #include "spesh/log.h"
 #include "spesh/threshold.h"
 #include "spesh/inline.h"
 #include "spesh/osr.h"
+#include "spesh/iterator.h"
 #include "spesh/lookup.h"
+#include "spesh/worker.h"
+#include "spesh/stats.h"
+#include "spesh/plan.h"
+#include "spesh/arg_guard.h"
+#include "strings/nfg.h"
 #include "strings/normalize.h"
 #include "strings/decode_stream.h"
 #include "strings/ascii.h"
+#include "strings/parse_num.h"
 #include "strings/utf8.h"
 #include "strings/utf8_c8.h"
 #include "strings/utf16.h"
-#include "strings/nfg.h"
 #include "strings/iter.h"
 #include "strings/ops.h"
 #include "strings/unicode_gen.h"
@@ -167,8 +176,6 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "io/io.h"
 #include "io/eventloop.h"
 #include "io/syncfile.h"
-#include "io/syncstream.h"
-#include "io/syncpipe.h"
 #include "io/syncsocket.h"
 #include "io/fileops.h"
 #include "io/dirops.h"
@@ -183,13 +190,19 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "core/intcache.h"
 #include "core/fixedsizealloc.h"
 #include "jit/graph.h"
+#include "jit/label.h"
+#include "jit/expr.h"
+#include "jit/register.h"
+#include "jit/tile.h"
 #include "jit/compile.h"
 #include "jit/log.h"
 #include "profiler/instrument.h"
 #include "profiler/log.h"
 #include "profiler/profile.h"
 #include "profiler/heapsnapshot.h"
+#include "profiler/telemeh.h"
 #include "instrument/crossthreadwrite.h"
+#include "instrument/line_coverage.h"
 
 MVMObject *MVM_backend_config(MVMThreadContext *tc);
 
