@@ -4,17 +4,17 @@ struct MVMUnicodeNamedValue {
     const char *name;
     MVMint32 value;
 };
-#define num_unicode_property_keypairs 3806
+#define num_unicode_property_keypairs 3144
 
-#define MVM_NUM_UNICODE_EXTENTS 106
+#define MVM_NUM_UNICODE_EXTENTS 107
 
 MVMint32 MVM_unicode_is_in_block(MVMThreadContext *tc, MVMString *str, MVMint64 pos, MVMString *block_name);
 
-#define MVM_CODEPOINT_NAMES_COUNT 52102
-#define MVM_NUM_PROPERTY_CODES 105
+#define MVM_CODEPOINT_NAMES_COUNT 52917
+#define MVM_NUM_PROPERTY_CODES 107
 
-#define num_unicode_seq_keypairs 1731
-#define num_unicode_namealias_keypairs 459
+#define num_unicode_seq_keypairs 1939
+#define num_unicode_namealias_keypairs 464
 struct MVMUnicodeNamedAlias {
     char *name;
     MVMGrapheme32 codepoint;
@@ -22,38 +22,38 @@ struct MVMUnicodeNamedAlias {
 };
 typedef struct MVMUnicodeNamedAlias MVMUnicodeNamedAlias;
 
-#define num_unicode_property_value_keypairs 3491
+#define num_unicode_property_value_keypairs 3608
 
 typedef enum {
     MVM_UNICODE_PROPERTY_DECOMP_SPEC = 1,
     MVM_UNICODE_PROPERTY_CASE_CHANGE_INDEX = 2,
-    MVM_UNICODE_PROPERTY_SPECIAL_CASING = 3,
+    MVM_UNICODE_PROPERTY_JOINING_GROUP = 3,
     MVM_UNICODE_PROPERTY_CASE_FOLDING = 4,
     MVM_UNICODE_PROPERTY_BLOCK = 5,
-    MVM_UNICODE_PROPERTY_SCRIPT = 6,
+    MVM_UNICODE_PROPERTY_NUMERIC_VALUE = 6,
     MVM_UNICODE_PROPERTY_SENTENCE_BREAK = 7,
-    MVM_UNICODE_PROPERTY_NUMERIC_VALUE = 8,
-    MVM_UNICODE_PROPERTY_JOINING_GROUP = 9,
-    MVM_UNICODE_PROPERTY_NUMERIC_VALUE_NUMERATOR = 10,
-    MVM_UNICODE_PROPERTY_LINE_BREAK = 11,
-    MVM_UNICODE_PROPERTY_JOINING_TYPE = 12,
+    MVM_UNICODE_PROPERTY_SCRIPT = 8,
+    MVM_UNICODE_PROPERTY_NUMERIC_VALUE_NUMERATOR = 9,
+    MVM_UNICODE_PROPERTY_SPECIAL_CASING = 10,
+    MVM_UNICODE_PROPERTY_CANONICAL_COMBINING_CLASS = 11,
+    MVM_UNICODE_PROPERTY_EAST_ASIAN_WIDTH = 12,
     MVM_UNICODE_PROPERTY_DIGIT = 13,
     MVM_UNICODE_PROPERTY_ASCII_HEX_DIGIT = 14,
-    MVM_UNICODE_PROPERTY_CANONICAL_COMBINING_CLASS = 15,
-    MVM_UNICODE_PROPERTY_GRAPHEME_CLUSTER_BREAK = 16,
-    MVM_UNICODE_PROPERTY_AGE = 17,
-    MVM_UNICODE_PROPERTY_GENERAL_CATEGORY = 18,
-    MVM_UNICODE_PROPERTY_NUMERIC_VALUE_DENOMINATOR = 19,
-    MVM_UNICODE_PROPERTY_DECOMPOSITION_TYPE = 20,
+    MVM_UNICODE_PROPERTY_LINE_BREAK = 15,
+    MVM_UNICODE_PROPERTY_AGE = 16,
+    MVM_UNICODE_PROPERTY_BIDI_CLASS = 17,
+    MVM_UNICODE_PROPERTY_DECOMPOSITION_TYPE = 18,
+    MVM_UNICODE_PROPERTY_GENERAL_CATEGORY = 19,
+    MVM_UNICODE_PROPERTY_GRAPHEME_CLUSTER_BREAK = 20,
     MVM_UNICODE_PROPERTY_ALPHABETIC = 21,
-    MVM_UNICODE_PROPERTY_WORD_BREAK = 22,
-    MVM_UNICODE_PROPERTY_BIDI_CLASS = 23,
-    MVM_UNICODE_PROPERTY_EAST_ASIAN_WIDTH = 24,
-    MVM_UNICODE_PROPERTY_HANGUL_SYLLABLE_TYPE = 25,
-    MVM_UNICODE_PROPERTY_NUMERIC_TYPE = 26,
-    MVM_UNICODE_PROPERTY_NFC_QC = 27,
+    MVM_UNICODE_PROPERTY_NUMERIC_VALUE_DENOMINATOR = 22,
+    MVM_UNICODE_PROPERTY_WORD_BREAK = 23,
+    MVM_UNICODE_PROPERTY_HANGUL_SYLLABLE_TYPE = 24,
+    MVM_UNICODE_PROPERTY_JOINING_TYPE = 25,
+    MVM_UNICODE_PROPERTY_NFC_QC = 26,
+    MVM_UNICODE_PROPERTY_NFG_QC = 27,
     MVM_UNICODE_PROPERTY_NFKC_QC = 28,
-    MVM_UNICODE_PROPERTY_NFG_QC = 29,
+    MVM_UNICODE_PROPERTY_NUMERIC_TYPE = 29,
     MVM_UNICODE_PROPERTY_ANY = 30,
     MVM_UNICODE_PROPERTY_ASSIGNED = 31,
     MVM_UNICODE_PROPERTY_BIDI_CONTROL = 32,
@@ -74,63 +74,98 @@ typedef enum {
     MVM_UNICODE_PROPERTY_DEPRECATED = 47,
     MVM_UNICODE_PROPERTY_DIACRITIC = 48,
     MVM_UNICODE_PROPERTY_EMOJI = 49,
-    MVM_UNICODE_PROPERTY_EMOJI_MODIFIER = 50,
-    MVM_UNICODE_PROPERTY_EMOJI_MODIFIER_BASE = 51,
-    MVM_UNICODE_PROPERTY_EMOJI_PRESENTATION = 52,
-    MVM_UNICODE_PROPERTY_EXTENDER = 53,
-    MVM_UNICODE_PROPERTY_FULL_COMPOSITION_EXCLUSION = 54,
-    MVM_UNICODE_PROPERTY_GRAPHEME_BASE = 55,
-    MVM_UNICODE_PROPERTY_GRAPHEME_EXTEND = 56,
-    MVM_UNICODE_PROPERTY_GRAPHEME_LINK = 57,
-    MVM_UNICODE_PROPERTY_HEX_DIGIT = 58,
-    MVM_UNICODE_PROPERTY_HYPHEN = 59,
-    MVM_UNICODE_PROPERTY_IDS_BINARY_OPERATOR = 60,
-    MVM_UNICODE_PROPERTY_IDS_TRINARY_OPERATOR = 61,
-    MVM_UNICODE_PROPERTY_ID_CONTINUE = 62,
-    MVM_UNICODE_PROPERTY_ID_START = 63,
-    MVM_UNICODE_PROPERTY_IDEOGRAPHIC = 64,
-    MVM_UNICODE_PROPERTY_JOIN_CONTROL = 65,
-    MVM_UNICODE_PROPERTY_L = 66,
-    MVM_UNICODE_PROPERTY_LC = 67,
-    MVM_UNICODE_PROPERTY_LOGICAL_ORDER_EXCEPTION = 68,
-    MVM_UNICODE_PROPERTY_LOWERCASE = 69,
-    MVM_UNICODE_PROPERTY_M = 70,
-    MVM_UNICODE_PROPERTY_MVM_COLLATION_QC = 71,
-    MVM_UNICODE_PROPERTY_MVM_COLLATION_SECONDARY = 72,
-    MVM_UNICODE_PROPERTY_MATH = 73,
-    MVM_UNICODE_PROPERTY_N = 74,
-    MVM_UNICODE_PROPERTY_NFD_QC = 75,
-    MVM_UNICODE_PROPERTY_NFKD_QC = 76,
-    MVM_UNICODE_PROPERTY_NONCHARACTER_CODE_POINT = 77,
+    MVM_UNICODE_PROPERTY_EMOJI_COMPONENT = 50,
+    MVM_UNICODE_PROPERTY_EMOJI_MODIFIER = 51,
+    MVM_UNICODE_PROPERTY_EMOJI_MODIFIER_BASE = 52,
+    MVM_UNICODE_PROPERTY_EMOJI_PRESENTATION = 53,
+    MVM_UNICODE_PROPERTY_EXTENDER = 54,
+    MVM_UNICODE_PROPERTY_FULL_COMPOSITION_EXCLUSION = 55,
+    MVM_UNICODE_PROPERTY_GRAPHEME_BASE = 56,
+    MVM_UNICODE_PROPERTY_GRAPHEME_EXTEND = 57,
+    MVM_UNICODE_PROPERTY_GRAPHEME_LINK = 58,
+    MVM_UNICODE_PROPERTY_HEX_DIGIT = 59,
+    MVM_UNICODE_PROPERTY_HYPHEN = 60,
+    MVM_UNICODE_PROPERTY_IDS_BINARY_OPERATOR = 61,
+    MVM_UNICODE_PROPERTY_IDS_TRINARY_OPERATOR = 62,
+    MVM_UNICODE_PROPERTY_ID_CONTINUE = 63,
+    MVM_UNICODE_PROPERTY_ID_START = 64,
+    MVM_UNICODE_PROPERTY_IDEOGRAPHIC = 65,
+    MVM_UNICODE_PROPERTY_JOIN_CONTROL = 66,
+    MVM_UNICODE_PROPERTY_L = 67,
+    MVM_UNICODE_PROPERTY_LC = 68,
+    MVM_UNICODE_PROPERTY_LOGICAL_ORDER_EXCEPTION = 69,
+    MVM_UNICODE_PROPERTY_LOWERCASE = 70,
+    MVM_UNICODE_PROPERTY_M = 71,
+    MVM_UNICODE_PROPERTY_MVM_COLLATION_QC = 72,
+    MVM_UNICODE_PROPERTY_MVM_COLLATION_SECONDARY = 73,
+    MVM_UNICODE_PROPERTY_MATH = 74,
+    MVM_UNICODE_PROPERTY_N = 75,
+    MVM_UNICODE_PROPERTY_NFD_QC = 76,
+    MVM_UNICODE_PROPERTY_NFKD_QC = 77,
     MVM_UNICODE_PROPERTY_MVM_COLLATION_PRIMARY = 78,
     MVM_UNICODE_PROPERTY_MVM_COLLATION_TERTIARY = 79,
-    MVM_UNICODE_PROPERTY_OTHER_ALPHABETIC = 80,
-    MVM_UNICODE_PROPERTY_OTHER_DEFAULT_IGNORABLE_CODE_POINT = 81,
-    MVM_UNICODE_PROPERTY_OTHER_GRAPHEME_EXTEND = 82,
-    MVM_UNICODE_PROPERTY_OTHER_ID_CONTINUE = 83,
-    MVM_UNICODE_PROPERTY_OTHER_ID_START = 84,
-    MVM_UNICODE_PROPERTY_OTHER_LOWERCASE = 85,
-    MVM_UNICODE_PROPERTY_OTHER_MATH = 86,
-    MVM_UNICODE_PROPERTY_OTHER_UPPERCASE = 87,
-    MVM_UNICODE_PROPERTY_P = 88,
-    MVM_UNICODE_PROPERTY_PATTERN_SYNTAX = 89,
-    MVM_UNICODE_PROPERTY_PATTERN_WHITE_SPACE = 90,
-    MVM_UNICODE_PROPERTY_PREPENDED_CONCATENATION_MARK = 91,
-    MVM_UNICODE_PROPERTY_QUOTATION_MARK = 92,
-    MVM_UNICODE_PROPERTY_RADICAL = 93,
-    MVM_UNICODE_PROPERTY_S = 94,
-    MVM_UNICODE_PROPERTY_SENTENCE_TERMINAL = 95,
-    MVM_UNICODE_PROPERTY_SOFT_DOTTED = 96,
-    MVM_UNICODE_PROPERTY_TERMINAL_PUNCTUATION = 97,
-    MVM_UNICODE_PROPERTY_UNIFIED_IDEOGRAPH = 98,
-    MVM_UNICODE_PROPERTY_UPPERCASE = 99,
-    MVM_UNICODE_PROPERTY_VARIATION_SELECTOR = 100,
-    MVM_UNICODE_PROPERTY_WHITE_SPACE = 101,
-    MVM_UNICODE_PROPERTY_XID_CONTINUE = 102,
-    MVM_UNICODE_PROPERTY_XID_START = 103,
-    MVM_UNICODE_PROPERTY_Z = 104,
+    MVM_UNICODE_PROPERTY_NONCHARACTER_CODE_POINT = 80,
+    MVM_UNICODE_PROPERTY_OTHER_ALPHABETIC = 81,
+    MVM_UNICODE_PROPERTY_OTHER_DEFAULT_IGNORABLE_CODE_POINT = 82,
+    MVM_UNICODE_PROPERTY_OTHER_GRAPHEME_EXTEND = 83,
+    MVM_UNICODE_PROPERTY_OTHER_ID_CONTINUE = 84,
+    MVM_UNICODE_PROPERTY_OTHER_ID_START = 85,
+    MVM_UNICODE_PROPERTY_OTHER_LOWERCASE = 86,
+    MVM_UNICODE_PROPERTY_OTHER_MATH = 87,
+    MVM_UNICODE_PROPERTY_OTHER_UPPERCASE = 88,
+    MVM_UNICODE_PROPERTY_P = 89,
+    MVM_UNICODE_PROPERTY_PATTERN_SYNTAX = 90,
+    MVM_UNICODE_PROPERTY_PATTERN_WHITE_SPACE = 91,
+    MVM_UNICODE_PROPERTY_PREPENDED_CONCATENATION_MARK = 92,
+    MVM_UNICODE_PROPERTY_QUOTATION_MARK = 93,
+    MVM_UNICODE_PROPERTY_RADICAL = 94,
+    MVM_UNICODE_PROPERTY_REGIONAL_INDICATOR = 95,
+    MVM_UNICODE_PROPERTY_S = 96,
+    MVM_UNICODE_PROPERTY_SENTENCE_TERMINAL = 97,
+    MVM_UNICODE_PROPERTY_SOFT_DOTTED = 98,
+    MVM_UNICODE_PROPERTY_TERMINAL_PUNCTUATION = 99,
+    MVM_UNICODE_PROPERTY_UNIFIED_IDEOGRAPH = 100,
+    MVM_UNICODE_PROPERTY_UPPERCASE = 101,
+    MVM_UNICODE_PROPERTY_VARIATION_SELECTOR = 102,
+    MVM_UNICODE_PROPERTY_WHITE_SPACE = 103,
+    MVM_UNICODE_PROPERTY_XID_CONTINUE = 104,
+    MVM_UNICODE_PROPERTY_XID_START = 105,
+    MVM_UNICODE_PROPERTY_Z = 106,
 } MVM_unicode_property_codes;
 
+/* MVM_UNICODE_PROPERTY_GENERAL_CATEGORY */
+#define MVM_UNICODE_PVALUE_GC_CC 15
+#define MVM_UNICODE_PVALUE_GC_CF 16
+#define MVM_UNICODE_PVALUE_GC_CN 0
+#define MVM_UNICODE_PVALUE_GC_CO 17
+#define MVM_UNICODE_PVALUE_GC_CS 18
+#define MVM_UNICODE_PVALUE_GC_LL 2
+#define MVM_UNICODE_PVALUE_GC_LM 4
+#define MVM_UNICODE_PVALUE_GC_LO 5
+#define MVM_UNICODE_PVALUE_GC_LT 3
+#define MVM_UNICODE_PVALUE_GC_LU 1
+#define MVM_UNICODE_PVALUE_GC_MC 8
+#define MVM_UNICODE_PVALUE_GC_ME 7
+#define MVM_UNICODE_PVALUE_GC_MN 6
+#define MVM_UNICODE_PVALUE_GC_ND 9
+#define MVM_UNICODE_PVALUE_GC_NL 10
+#define MVM_UNICODE_PVALUE_GC_NO 11
+#define MVM_UNICODE_PVALUE_GC_PC 22
+#define MVM_UNICODE_PVALUE_GC_PD 19
+#define MVM_UNICODE_PVALUE_GC_PE 21
+#define MVM_UNICODE_PVALUE_GC_PF 29
+#define MVM_UNICODE_PVALUE_GC_PI 28
+#define MVM_UNICODE_PVALUE_GC_PO 23
+#define MVM_UNICODE_PVALUE_GC_PS 20
+#define MVM_UNICODE_PVALUE_GC_SC 25
+#define MVM_UNICODE_PVALUE_GC_SK 26
+#define MVM_UNICODE_PVALUE_GC_SM 24
+#define MVM_UNICODE_PVALUE_GC_SO 27
+#define MVM_UNICODE_PVALUE_GC_ZL 13
+#define MVM_UNICODE_PVALUE_GC_ZP 14
+#define MVM_UNICODE_PVALUE_GC_ZS 12
+
+
 /* MVM_UNICODE_PROPERTY_GRAPHEME_CLUSTER_BREAK */
 #define MVM_UNICODE_PVALUE_GCB_CR 2
 #define MVM_UNICODE_PVALUE_GCB_CONTROL 4
